﻿/*
 *Copyright(C) 2020 by  Cyf All rights reserved.
 *Unity版本：2021.3.11f1c2 
 *作者:程一峰  
 *创建日期: 2022-11-10 
 *模块说明：木蝴蝶数学库：数据格式
 *版本: 1.0
*/

using Unity.Mathematics;
using UnityEngine;

namespace Oroxylum
{
    /// <summary>
    /// 平面，与 UnityEngine.Plane 等价；
    /// 不用Unity的Plane是因为其是基于 Vector 构建的；
    /// 而 Plane3d 是基于 float3 构建
    /// </summary>
    public struct Plane3d
    {
        /// <summary>
        /// 平面法线方向，已经  normalize
        /// </summary>
        public float3 Normal;
        public float Distance;

        /// <summary>
        /// 平面上的一点；
        /// </summary>
        public float3 PlanePoint => Normal * -Distance;

        /// <summary>
        /// 构造平面
        /// </summary>
        /// <param name="normal">法线</param>
        /// <param name="distance">平面到原点的距离</param>
        public Plane3d(float3 normal, float distance)
        {
            Normal = math.normalize(normal);
            Distance = distance;
        }

        /// <summary>
        /// 构造平面
        /// </summary>
        /// <param name="normal">法线</param>
        /// <param name="point">平面上一点</param>
        public Plane3d(float3 normal, float3 point)
        {
            Normal = math.normalize(normal);
            Distance = -math.dot(Normal, point);
        }

        // N = Cross(P1-P0,P2-P0)/Length(Cross(P1-P0,P2-P0)), c = Dot(N,P0) where
        // P0, P1, P2 are points on the plane.
        /// <summary>
        /// 根据平面上的3个点进行构建；
        /// </summary>
        public Plane3d(float3 a, float3 b, float3 c)
        {
            Normal = math.normalize(math.cross(b - a, c - a));
            Distance = -math.dot(Normal, a);
        }

        public Plane3d(Plane p)
        {
            Normal = p.normal;
            Distance = p.distance;
        }

        // Compute d = Dot(N,P)-c where N is the plane normal and c is the plane
        // constant.  This is a signed distance.  The sign of the return value is
        // positive if the point is on the positive side of the plane, negative if
        // the point is on the negative side, and zero if the point is on the
        // plane.
        public float GetDistanceToPoint(float3 p)
        {
            return Normal.Dot(p) - Distance;
        }

        // The "positive side" of the plane is the half space to which the plane
        // normal points.  The "negative side" is the other half space.  The
        // function returns +1 when P is on the positive side, -1 when P is on the
        // the negative side, or 0 when P is on the plane.
        public int WhichSide(float3 p)
        {
            double distance = GetDistanceToPoint(p);
            if (distance < 0)
                return -1;
            else if (distance > 0)
                return +1;
            else
                return 0;
        }
        public bool SameSide(float3 inPt0, float3 inPt1)
        {
            float distanceToPoint = GetDistanceToPoint(inPt0);
            float distanceToPoint2 = GetDistanceToPoint(inPt1);
            return (distanceToPoint > 0f && distanceToPoint2 > 0f) || (distanceToPoint <= 0f && distanceToPoint2 <= 0f);
        }
    }
}
