﻿/*
 *Copyright(C) 2023 by GYYX All rights reserved.
 *Unity版本：2020.3.48f1 
 *作者:程一峰  
 *创建日期: 2023-06.16 
 *模块说明：多线程通用库
 *版本: 1.0
*/

using System.Collections.Concurrent;
using System.Threading;

namespace Oroxylum
{
    /// <summary>
    /// 受限制的对象池
    /// </summary>
    public class LimitConcurrentPool<T> : ConcurrentPool<T> where T : class, IConcurrentPoolItem, new()
    {

        private int m_Capacity = 4;

        /// <summary>
        /// 容量
        /// </summary>
        public int Capacity
        {
            get
            {
                return m_Capacity;
            }
            set
            {
                if (value <= 0)
                {
                    Logger.LogError($"不能将容量设置为 0 ！");
                    return;
                }

                if (value < m_Capacity)
                {
                    Logger.LogError($"容量需要大于 {m_Capacity} !");
                    return;
                }

                m_Capacity = value;
            }
        }

        public LimitConcurrentPool(int limitCount)
        {
            Capacity = limitCount;
            m_UsingItems = new ConcurrentQueue<T>();
        }

        //获取时加入堆栈
        private ConcurrentQueue<T> m_UsingItems;

        /// <summary>
        /// 使用中的数量
        /// </summary>
        private int m_UsingItemCount;
        public int UsingCount => m_UsingItemCount;

        public override T Get()
        {
            //查看当前已经使用的数量是否超过限制；
            if (m_UsingItemCount >= m_Capacity)
            {
                //先触发回收
                Recycle(100);//每次回收100个；

                //回收后仍然没有可用对象，表示对象池已耗尽；
                if (m_Bag.Count == 0)
                {
                    Interlocked.Increment(ref m_Capacity);
                    Logger.LogWarning($"[LimitConcurrentPool] 对象池已耗尽！容量：{m_Capacity}");
                }
            }

            //再取出一个
            T result = base.Get();
            //加到队列中；
            m_UsingItems.Enqueue(result);
            Interlocked.Increment(ref m_UsingItemCount);
            //返回结果
            return result;
        }

        /// <summary>
        /// 回收当前可回收的所有单位
        /// </summary>
        public void Recycle(int maxCount = int.MaxValue)
        {
            int usingCount = m_UsingItems.Count;
            int length = maxCount > usingCount ? usingCount : maxCount;
            for (int i = 0; i < length; i++)
            {
                if (m_UsingItems.TryDequeue(out T result))
                {
                    if (result.Recyclable)
                        Put(result);
                    else//如果不能回收，则将其放回到队列中；
                        m_UsingItems.Enqueue(result);
                }
            }
        }

        public override bool Put(T obj)
        {
            var ret = base.Put(obj);
            if (ret)
                Interlocked.Decrement(ref m_UsingItemCount);
            return ret;
        }

        public override void Clear()
        {
            base.Clear();
            m_UsingItemCount = 0;

            //m_UsingItems.Clear();Unity 2022+ 有 Clear 方法 netstandard, Version=2.1.0.0
            m_UsingItems = new ConcurrentQueue<T>();
        }

    }
}
