﻿/*
 *Copyright(C) 2023 by GYYX All rights reserved.
 *Unity版本：2020.3.48f1 
 *作者:程一峰  
 *创建日期: 2023-06.16 
 *模块说明：多线程通用库
 *版本: 1.0
*/

using System;
using System.Collections.Concurrent;
using System.Reflection;
using System.Threading.Tasks;

#if TASK_PROFILE
using UnityEngine.Profiling;
#endif

namespace Oroxylum
{
    public delegate void CallBackHandler();

    public static class TaskUtils
    {
        #region 安全的TPL启动

        public static Task Run(Action action)
        {
            if (action == null)
                return Task.CompletedTask;

            var task = Task.Run(() =>
            {
                try
                {
#if TASK_PROFILE
                    var method = action.Method;
                    if (method == null)
                        return;

                    Profiler.BeginThreadProfiling("GYTask", $"GYTask_{Task.CurrentId}");
                    Profiler.BeginSample(method.GetMethodName());
#endif
                    //在子线程运行全部任务，并等待完成以收集错误
                    var task = Task.Run(action);
                    task.Wait();

#if TASK_PROFILE
                    Profiler.EndSample();
                    Profiler.EndThreadProfiling();
#endif
                }
                catch (AggregateException ex)
                {
                    var method = action.Method;
                    LogException(ex, method.GetMethodName());
                }
            });

            return task;
        }

        public static Task Run(Func<Task> action)
        {
            if (action == null)
                return Task.CompletedTask;

            var task = Task.Run(() =>
            {
                try
                {
#if TASK_PROFILE
                    var method = action.Method;
                    if (method == null)
                        return;

                    Profiler.BeginThreadProfiling("GYTask", $"GYTask_{Task.CurrentId}");
                    Profiler.BeginSample(method.GetMethodName());
#endif
                    //在子线程运行全部任务，并等待完成以收集错误
                    var task = Task.Run(action);
                    task.Wait();

#if TASK_PROFILE
                    Profiler.EndSample();
                    Profiler.EndThreadProfiling();
#endif
                }
                catch (AggregateException ex)
                {
                    var method = action.Method;
                    LogException(ex, method.GetMethodName());
                }
            });

            return task;
        }

        private static void LogException(AggregateException aggregateException, string methodName)
        {
            string errorStr = "=====> 并行任务出错 <=====";
            errorStr = $"{errorStr}\n 错误方法：{methodName}\n Message: \n{aggregateException.Message} \n StackTrace: \n {aggregateException.StackTrace}";

            var exs = aggregateException.InnerExceptions;
            int length = exs.Count;
            for (int i = 0; i < length; i++)
            {
                var ex = exs[i];
                errorStr = $"{errorStr}\n InnerExceptions [{i}]: \n Message:\n{ex.Message}\nInnerException: \n{ex.InnerException}\nSource: \n {ex.Source} \n StackTrace: \n {ex.StackTrace}";
            }
            Logger.LogError(errorStr);
        }

        #endregion

        #region 主线程回调方法

        internal static ConcurrentQueue<CallBackHandler> MainThreadCallBackQueue = new ConcurrentQueue<CallBackHandler>();

        public static void AddMainThreadCallBack(CallBackHandler handler)
        {
            MainThreadCallBackQueue.Enqueue(handler);
        }

        /// <summary>
        /// 需要外部调用来控制刷新
        /// </summary>
        public static void UpdateMainThreadCallBack()
        {
            if (MainThreadCallBackQueue.Count == 0)
                return;

            while (MainThreadCallBackQueue.Count > 0)
            {
                MainThreadCallBackQueue.TryDequeue(out var handler);
                handler?.Invoke();
            }
        }


        #endregion

        #region 辅助方法

        private static string GetMethodName(this MethodInfo info)
        {
            if (info == null)
                return string.Empty;

            var type = info.DeclaringType;
            var typeNameSpace = type.Namespace;
            var typeName = type.Name;
            var methodName = info.Name;
            return $"{typeNameSpace}.{typeName}.{methodName}";
        }

        #endregion
    }
}